/*
 * PicStamp
 * A simple plugable and breadboardable board
 *
 * Copyright (C) 2012  B.Stultiens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdint.h>
#include <string.h>
#include <pic18fregs.h>
#include <clock.h>

#define clock_countdown(x)	do { if(clock_timers[x].state == TIMER_RUNNING) { \
					if(!--clock_timers[x].cnt) \
						clock_timers[x].state = TIMER_EXPIRED; \
					} \
				} while(0)

volatile clock_timer_t clock_timers[MAXTIMERS];
volatile uint32_t clock_secs;			/* Seconds (unix epoch if we base it) */
volatile uint16_t clock_msecs;			/* Millisecond resolution [0..999] */

void clock_init(void)
{
	clock_secs = 0;
	clock_msecs = 0;
	memset(clock_timers, 0, sizeof(clock_timers));
	PR4 = (100-1);
	TMR4 = 0;
	PIR5bits.TMR4IF = 0;
	T4CON = 0x4e;	/* Pre 1:16, TMR on, Post 1:10 */
	PIE5bits.TMR4IE = 1;
}

void clock_isr(void)
{
	PIR5bits.TMR4IF = 0;
	if(++clock_msecs == 1000) {
		clock_msecs = 0;
		clock_secs++;
	}

#if 0
	uint8_t i;
	for(i = 0; i < MAXTIMERS; i++) {
		if(clock_timers[i].state == TIMER_RUNNING) {
			if(!--clock_timers[i].cnt)
				clock_timers[i].state = TIMER_EXPIRED;
		}
	}
#else
	/* Loop-unroll is faster */
	/* XXX: But beware, changing the list must be reflected here */
	clock_countdown(TIMER_BLINKY);
	clock_countdown(TIMER_WHEEL);
#endif
}

