/*
 * PicStamp
 * A simple plugable and breadboardable board
 *
 * Copyright (C) 2012  B.Stultiens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __PICSTAMP_CLOCK_H
#define __PICSTAMP_CLOCK_H

#include <irq.h>

enum {
	TIMER_BLINKY = 0,
	TIMER_WHEEL,
	MAXTIMERS
};

enum {
	TIMER_IDLE = 0,
	TIMER_RUNNING,
	TIMER_EXPIRED
};

typedef struct __clock_timer_t {
	uint8_t		state;
	uint16_t	cnt;
} clock_timer_t;

extern volatile clock_timer_t clock_timers[MAXTIMERS];
extern volatile uint32_t clock_secs;
extern volatile uint16_t clock_msecs;

void clock_init(void);
void clock_isr(void);

#define clock_timer_set(id,ms)		do { cli(); clock_timers[id].cnt = ms; clock_timers[id].state = TIMER_RUNNING; sti(); } while(0)
#define clock_timer_set_isr(id,ms)	do { clock_timers[id].cnt = ms; clock_timers[id].state = TIMER_RUNNING; } while(0)
#define clock_timer_expired(id)		(clock_timers[id].state == TIMER_EXPIRED)
#define clock_timer_idle(id)		(clock_timers[id].state == TIMER_IDLE)
#define clock_timer_clear(id)		do { cli(); clock_timers[id].state = TIMER_IDLE; sti(); } while(0)
#define clock_timer_clear_isr(id)	do { clock_timers[id].state = TIMER_IDLE; } while(0)

#endif
